#include "mouse.h"

/*
	Reset the mouse driver.  Center the cursor and make it hidden.
	Also used to reset mouse at end of program.
	Exit conditions:
		exists==!0 if driver installed
		buttons==number of buttons
*/
void mouse::reset()
{
  inreg.x.ax = 0;
  call_mdd;
  exists=outreg.x.ax;
  buttons=outreg.x.bx;
}

/*
	If mouse cursor is invisible, turns it visible.
*/
void mouse::show()
{
  if (hiding)
  {
    hiding = 0;
    inreg.x.ax = 1;
    call_mdd;
  }
}

/*
	If mouse cursor is visible, hides it.
*/
void mouse::hide()
{
  if (!hiding)
  {
    hiding = 1;
    inreg.x.ax = 2;
    call_mdd;
  }
}

/*
	Report real-time mouse status.  Used in graphics to draw pixels
        in a loop.  Used also to determine cursor location.
*/
void mouse::getpos()
{
  inreg.x.ax=3;
  call_mdd;
  button_status=outreg.x.bx;
  column=outreg.x.cx;
  row=outreg.x.dx;
}

/*
	Give position in pixels; 640x200 for CGA 80x25 text.
	An automatic rounding is done in text mode.
*/
void mouse::setpos(int newcol, int newrow)
{
  inreg.x.ax=4;
  inreg.x.cx=newcol;
  inreg.x.dx=newrow;
  call_mdd;
}

/*
	Get pressed info about named button.
	op_count tells number of hits, button_status gives real_time button
	info and column,row give coords of mouse when button last hit.
	If status=1 for button then column,row are current cursor loc.
*/
void mouse::pressed(int button)
{
  inreg.x.ax=5;
  inreg.x.bx=button;
  call_mdd;
  button_status=outreg.x.ax;
  op_count=outreg.x.bx;
  column=outreg.x.cx;
  row=outreg.x.dx;
}

/*
	Same format as pressed(), yet more accurate since release is
	relatively instantaneous.
*/
void mouse::released(int button)
{
  inreg.x.ax=6;
  inreg.x.bx=button;
  call_mdd;
  button_status=outreg.x.ax;
  op_count=outreg.x.bx;
  column=outreg.x.cx;
  row=outreg.x.dx;
}

/*
	Set minimum, maximum ranges for x positions
	Moves cursor inside those ranges when called
        Swaps values if they are reversed
*/
void mouse::xrange(int min, int max)	//set valid ranges, restrict cursor to
{
  inreg.x.ax=7;
  inreg.x.cx=lower(min,max);
  inreg.x.dx=upper(min,max);
  call_mdd;
}

/*
	As previous but sets y-ranges
	example: pop-up:
		1. get cursor pos
		2. make pop-up
		3. set bounds
		4. do ops, remove pop-up
		5. restore full screen bounds
		6. set pos to previous
*/
void mouse::yrange(int min, int max)  //valid range if called
{
  inreg.x.ax=8;
  inreg.x.cx=lower(min,max);
  inreg.x.dx=upper(min,max);
  call_mdd;
}

/*
	Set graphic cursor shape
*/
void mouse::graph_cursor(int h_hot, int v_hot, unsigned mask_seg,
			 unsigned mask_ofs)
{
  struct SREGS seg;

  inreg.x.ax=9;
  inreg.x.bx=h_hot;	//horizontal hotspot
  inreg.x.cx=v_hot;	//vertical hotspot
  inreg.x.dx=mask_ofs;
  seg.es=mask_seg;
  int86x(0x33,&inreg,&outreg,&seg);
}

/*
	cursor type: 0 = software (default)
		     1 = hardware
	software cursor: arg1, arg2 are cursor masks
	hardware cursor: arg1, arg2 are start,stop scan lines

	hardware cursor is video cursor (single cursor)
		text i/o position does not move
	software cursor: two cursors, one is i/o,
		other is mouse
		s. c. can use any of 256 text chars
		default is full-cell block that inverts video
		does not blink
		arg1:
			0x77ff if see-thru rectangle
			0 if uses text chars
		arg2:
			fg/bg attrib in upper eight bits
			ASCII in lower eight bits
		ex:
			rectangle: 0x77ff,0x7700
			grey  on black background: 0x0000,0x0718
	hardware: 8 scans per cursor in text mode, 12 for mono		
*/
void mouse::text_cursor(int c_type, unsigned arg1, unsigned arg2)
{
  inreg.x.ax=10;
  inreg.x.bx=c_type;
  inreg.x.cx=arg1;
  inreg.x.dx=arg2;
  call_mdd;
}

/*
	Get net horizontal and vertical motion
*/
void mouse::motion()
{
  inreg.x.ax=11;
  call_mdd;
  hcount=outreg.x.cx;
  vcount=_DX;
}

/*
  Function 12: define mouse event handler
  Attach a customized task to mouse device driver.
  When event occurs, mouse driver calls task.

  Event: defined by mask.
  Mask: 16 bit word
  Bit		Event
  0		Mouse moved
  1		left pressed
  2		left released
  3		right pressed
  4		right released
  5		center pressed
  6		center released

  Info passed to handler like so:
  ax		event that occurred, set as above
  bx		button status
  cx		horizontal position
  dx		vertical position

  Event handler like interrupt but must exit with far return.
  Cannot perform DOS or BIOS calls or I/O, but should update a record
  which can be checked.
*/
void mouse::install_task(unsigned mask, unsigned task_seg, unsigned task_ofs)
{
  struct SREGS seg;

  inreg.x.ax=12;
  inreg.x.cx=mask;
  inreg.x.dx=task_ofs;
  seg.es=task_seg;
  int86x(0x33,&inreg,&outreg,&seg);
}

void mouse::pen_on()
{
  inreg.x.ax=13;
  call_mdd;
}

void mouse::pen_off()
{
  inreg.x.ax=14;
  call_mdd;
}

void mouse::ratio(int horiz, int vert)
{
  inreg.x.ax=15;
  inreg.x.cx=horiz;
  inreg.x.dx=vert;
  call_mdd;
}
